<?php

require "../config/config.php";

// Eine minimale Wrapper-Klasse für mysqli-Funktionen
// Diese Klasse bietet eine einfache Schnittstelle für Datenbankoperationen
// mit mysqli und verwendet nur Prepared Statements. Sie enthält auch
// Methoden zum Abrufen von Abfrageergebnissen.
class Database {
    private mysqli $connection;

    // Konstruktor - stellt eine Datenbankverbindung her
    // Wirft eine Exception, wenn die Verbindung fehlschlägt
    public function __construct() {
        // Verbindung mit Konstanten aus der Konfiguration erstellen
        $this->connection = new mysqli(DB_SERVER, DB_USERNAME, DB_PASSWORD, DB_NAME);

        // Verbindung überprüfen
        if ($this->connection->connect_error) {
            throw new Exception("Connection failed: " . $this->connection->connect_error);
        }

        $this->initialize();
    }

    // Führt eine Abfrage mit Prepared Statements aus
    // $sql: Die auszuführende SQL-Abfrage
    // $params: Parameter für das Prepared Statement
    // Gibt ein mysqli_result-Objekt oder einen booleschen Wert zurück
    // Wirft eine Exception, wenn die Abfrage fehlschlägt
    public function query(string $sql, array $params = []): mysqli_result|bool {
        // Prepared Statement mit Parametern
        $stmt = $this->connection->prepare($sql);

        if ($stmt === false) {
            throw new Exception("Prepare failed: " . $this->connection->error);
        }

        // Bestimme den Typen-String für bind_param
        $types = '';
        foreach ($params as $param) {
            if (is_int($param)) {
                $types .= 'i';
            } elseif (is_float($param)) {
                $types .= 'd';
            } elseif (is_string($param)) {
                $types .= 's';
            } else {
                $types .= 'b';
            }
        }

        // Parameter binden
        if (!empty($params)) {
            $stmt->bind_param($types, ...$params);
        }

        // Statement ausführen
        if (!$stmt->execute()) {
            throw new Exception("Execute failed: " . $stmt->error);
        }

        $result = $stmt->get_result();
        $stmt->close();

        return $result ?: true;
    }

    // Holt alle Zeilen aus einem Ergebnissatz als assoziatives Array
    // $result: Der Ergebnissatz
    // Gibt ein Array von Zeilen zurück
    public function fetchAll(mysqli_result $result): array {
        return $result->fetch_all(MYSQLI_ASSOC);
    }

    // Holt eine einzelne Zeile aus einem Ergebnissatz als assoziatives Array
    // $result: Der Ergebnissatz
    // Gibt die Zeile als assoziatives Array zurück oder null, wenn keine Zeilen vorhanden sind
    public function fetchOne(mysqli_result $result): ?array {
        $row = $result->fetch_assoc();
        return $row ?: null;
    }

    // Initialisiert das Datenbankschema
    // 
    // Erstellt die notwendigen Tabellen, falls sie nicht existieren:
    // - user: id, name, email, telephone
    // - category: id, name
    // - advert: id, name, description, date, user_id
    // - advert_category: advert_id, category_id (Viele-zu-viele-Beziehung)
    // 
    // Gibt true zurück, wenn die Initialisierung erfolgreich war
    // Wirft eine Exception, wenn die Initialisierung fehlschlägt
    public function initialize(): bool {
        try {
            // Erstelle Benutzer-Tabelle, falls sie nicht existiert
            $this->query("
                CREATE TABLE IF NOT EXISTS user (
                    id INT AUTO_INCREMENT PRIMARY KEY,
                    name VARCHAR(255) NOT NULL,
                    email VARCHAR(255) NOT NULL,
                    telephone VARCHAR(50) NOT NULL
                )
            ");

            // Erstelle Kategorie-Tabelle, falls sie nicht existiert
            $this->query("
                CREATE TABLE IF NOT EXISTS category (
                    id INT AUTO_INCREMENT PRIMARY KEY,
                    name VARCHAR(255) NOT NULL
                )
            ");

            // Erstelle Anzeigen-Tabelle, falls sie nicht existiert
            $this->query("
                CREATE TABLE IF NOT EXISTS advert (
                    id INT AUTO_INCREMENT PRIMARY KEY,
                    name VARCHAR(255) NOT NULL,
                    description VARCHAR(255) NOT NULL,
                    date DATETIME NOT NULL,
                    user_id INT NOT NULL,
                    FOREIGN KEY (user_id) REFERENCES user(id)
                )
            ");

            // Erstelle Anzeigen-Kategorie-Tabelle, falls sie nicht existiert (Viele-zu-viele-Beziehung)
            $this->query("
                CREATE TABLE IF NOT EXISTS advert_category (
                    advert_id INT NOT NULL,
                    category_id INT NOT NULL,
                    PRIMARY KEY (advert_id, category_id),
                    FOREIGN KEY (advert_id) REFERENCES advert(id),
                    FOREIGN KEY (category_id) REFERENCES category(id)
                )
            ");

            return true;
        } catch (Exception $e) {
            throw new Exception("Schema initialization failed: " . $e->getMessage());
        }
    }
}
